@@ -12,6 +12,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import sys
+
 from os_client_config.config import OpenStackConfig  # noqa
 
 
@@ -30,3 +32,24 @@ def simple_client(service_key, cloud=None, region_name=None):
     """
     return OpenStackConfig().get_one_cloud(
         cloud=cloud, region_name=region_name).get_session_client('compute')
+
+
+def make_client(service_key, constructor, options=None, **kwargs):
+    """Simple wrapper for getting a client instance from a client lib.
+
+    OpenStack Client Libraries all have a fairly consistent constructor
+    interface which os-client-config supports. In the simple case, there
+    is one and only one right way to construct a client object. If as a user
+    you don't want to do fancy things, just use this. It honors OS_ environment
+    variables and clouds.yaml - and takes as **kwargs anything you'd expect
+    to pass in.
+    """
+    config = OpenStackConfig()
+    if options:
+        config.register_argparse_options(options, sys.argv, service_key)
+        parsed_options = options.parse_args(sys.argv)
+    else:
+        parsed_options = None
+
+    cloud_config = config.get_one_cloud(options=parsed_options, **kwargs)
+    return cloud_config.get_legacy_client(service_key, constructor)
